@@ -127,6 +127,9 @@ int main(int argc, char **argv)
 	patch(1, "bvec_kmap_local", true, false,
 	      COMPAT_HAVE_BVEC_KMAP_LOCAL, "present");
 
+	patch(1, "sendpage", false, true,
+	      COMPAT_HAVE_SENDPAGE, "present");
+
 	patch(1, "struct_bvec_iter", true, false,
 	      COMPAT_HAVE_STRUCT_BVEC_ITER, "present");
 
@@ -150,8 +153,15 @@ int main(int argc, char **argv)
 	patch(1, "blk_cleanup_disk", false, true,
 	      COMPAT_HAVE_BLK_CLEANUP_DISK, "present");
 
+	patch(1, "queue_limits_features", true, false,
+	      COMPAT_QUEUE_LIMITS_HAS_FEATURES, "present");
+
+	patch(1, "blk_alloc_disk", true, false,
+	      COMPAT_BLK_ALLOC_DISK_TAKES_QUEUE_LIMITS, "takes_queue_limits");
+#if !defined(COMPAT_BLK_ALLOC_DISK_TAKES_QUEUE_LIMITS)
 	patch(1, "blk_alloc_disk", true, false,
 	      COMPAT_HAVE_BLK_ALLOC_DISK, "present");
+#endif
 
 /*******************************************************************************/
 	/*
@@ -260,13 +270,13 @@ int main(int argc, char **argv)
 	patch(1, "security_netlink_recv", false, true,
 	      COMPAT_HAVE_SECURITY_NETLINK_RECV, "present");
 
-#if defined(COMPAT_HAVE_QUEUE_FLAG_STABLE_WRITES)
-	/* in versions >=5.9, there is QUEUE_FLAG_STABLE_WRITES */
-#else
-	/* for <5.9 but >=3.9, fall back to BDI_CAP_STABLE_WRITES */
+	/*
+	 * >= 6.10:  BLK_FEAT_STABLE_WRITES
+	 * 5.9-6.10: QUEUE_FLAG_STABLE_WRITES
+	 * <5.9:     BDI_CAP_STABLE_WRITES
+	 */
 	patch(1, "queue_flag_stable_writes", true, false,
-	      NO, "present");
-#endif
+	      COMPAT_HAVE_QUEUE_FLAG_STABLE_WRITES, "present");
 
 	patch(1, "queue_flag_discard", false, true,
 	      COMPAT_HAVE_QUEUE_FLAG_DISCARD, "present");
@@ -386,6 +396,12 @@ int main(int argc, char **argv)
 	      COMPAT_HAVE_WB_CONGESTED_ENUM, "present");
 #endif
 
+	patch(1, "queue_limits_start_update", true, false,
+	      COMPAT_HAVE_QUEUE_LIMITS_START_UPDATE, "present");
+
+	patch(1, "queue_limits", true, false,
+	      COMPAT_QUEUE_LIMITS_HAS_MAX_HW_DISCARD_SECTORS, "has_max_hw_discard_sectors");
+
 	patch(1, "disk_update_readahead", true, false,
 	      COMPAT_HAVE_DISK_UPDATE_READAHEAD, "present");
 
@@ -468,6 +484,9 @@ int main(int argc, char **argv)
 	      NO, "is_block_device");
 #endif
 
+	patch(1, "bio_max_segs", true, false,
+	      COMPAT_HAVE_BIO_MAX_SEGS, "present");
+
 	patch(1, "bio_max_vecs", true, false,
 	      COMPAT_HAVE_BIO_MAX_VECS, "present");
 
@@ -529,8 +548,10 @@ int main(int argc, char **argv)
 	patch(1, "kvfree_rcu_mightsleep", true, false,
 	      COMPAT_HAVE_KVFREE_RCU_MIGHTSLEEP, "present");
 
+#if !defined(COMPAT_HAVE_KVFREE_RCU_MIGHTSLEEP)
 	patch(1, "kvfree_rcu", true, false,
 	      COMPAT_HAVE_KVFREE_RCU, "present");
+#endif
 
 	patch(1, "list_next_entry", true, false,
 	      COMPAT_HAVE_LIST_NEXT_ENTRY, "present");
@@ -556,6 +577,46 @@ int main(int argc, char **argv)
 	patch(1, "__bio_add_page", true, false,
 	      COMPAT_HAVE___BIO_ADD_PAGE, "present");
 
+	patch(1, "blk_mode_t", true, false,
+	      COMPAT_HAVE_BLK_MODE_T, "present");
+
+	/* blkdev_put is oldest, then bdev_open_by_path, then bdev_file_open_by_path */
+	patch(1, "bdev_file_open_by_path", true, false,
+	      COMPAT_HAVE_BDEV_FILE_OPEN_BY_PATH, "present");
+#if !defined(COMPAT_HAVE_BDEV_FILE_OPEN_BY_PATH)
+	/* middle ground: bdev_open_by_path */
+	patch(1, "bdev_open_by_path", true, false,
+	      COMPAT_HAVE_BDEV_OPEN_BY_PATH, "present");
+# if !defined(COMPAT_HAVE_BDEV_OPEN_BY_PATH)
+	/* old: blkdev_get_by_* and blkdev_put */
+	patch(1, "blkdev_put", true, false,
+	      COMPAT_BLKDEV_PUT_HAS_HOLDER, "has_holder");
+	patch(1, "blkdev_get_by_path", true, false,
+	      COMPAT_BLKDEV_GET_BY_PATH_HAS_HOLDER_OPS, "has_holder_ops");
+# endif
+#endif
+
+	patch(1, "block_device_operations_open", true, false,
+	      COMPAT_BLOCK_DEVICE_OPERATIONS_OPEN_TAKES_GENDISK, "takes_gendisk");
+
+	patch(1, "block_device_operations_release", true, false,
+	      COMPAT_BLOCK_DEVICE_OPERATIONS_RELEASE_TAKES_SINGLE_ARGUMENT, "takes_single_argument");
+
+	patch(1, "genl_info_userhdr", true, false,
+	      COMPAT_HAVE_GENL_INFO_USERHDR, "present");
+
+	patch(1, "need_recalc_sigpending", false, true,
+		COMPAT_NEED_RECALC_SIGPENDING, "need");
+
+	patch(1, "counted_by", true, false,
+		COMPAT_HAVE_COUNTED_BY, "present");
+
+	patch(1, "linux_unaligned_h", true, false,
+		COMPAT_HAVE_LINUX_UNALIGNED_H, "present");
+
+	patch(1, "genlmsg_multicast_allns", true, false,
+	      COMPAT_GENLMSG_MULTICAST_ALLNS_HAS_4_PARAMS, "has_4_params");
+
 /* #define BLKDEV_ISSUE_ZEROOUT_EXPORTED */
 /* #define BLKDEV_ZERO_NOUNMAP */
 
