@@ -1,6 +1,7 @@
 #include <asm/param.h>                  /* for HZ */
 #include <linux/time.h>                 /* for NSECS_PER_SEC */
 
+/* If it is not exported yet, use our own copy. */
 static u64 nsecs_to_jiffies64(u64 n)
 {
 #if (NSEC_PER_SEC % HZ) == 0
@@ -18,6 +19,11 @@ static u64 nsecs_to_jiffies64(u64 n)
 #endif
 }
 
+/* Ged rid of "missing prototype" warning.
+ * The real prototype is in linux/jiffies.h,
+ * which we do not include here, for reasons I do not recall. */
+unsigned long nsecs_to_jiffies(u64 n);
+
 unsigned long nsecs_to_jiffies(u64 n)
 {
 	return (unsigned long)nsecs_to_jiffies64(n);
