@@ -36,12 +36,16 @@ void *drbd_md_get_buffer(struct drbd_device *device, const char *intent)
 			device->disk_state[NOW] <= D_FAILED,
 			HZ * 10);
 
-	if (t == 0)
-		drbd_err(device, "Waited 10 Seconds for md_buffer! BUG?\n");
-
 	if (r) {
-		drbd_err(device, "Failed to get md_buffer for %s, currently in use by %s\n",
-			 intent, device->md_io.current_use);
+		if (t == 0) {
+			drbd_err(device, "Waited 10 Seconds for md_buffer! BUG?\n");
+			drbd_err(device, "Failed to get md_buffer for %s, currently in use by %s\n",
+				 intent, device->md_io.current_use);
+		} else {
+			drbd_err(device, "Failed to get md_buffer for %s: disk state %s\n",
+				 intent, drbd_disk_str(device->disk_state[NOW]));
+		}
+
 		return NULL;
 	}
 
@@ -70,7 +74,11 @@ void wait_until_done_or_force_detached(struct drbd_device *device, struct drbd_b
 		dt = MAX_SCHEDULE_TIMEOUT;
 
 	dt = wait_event_timeout(device->misc_wait,
-			*done || test_bit(FORCE_DETACH, &device->flags), dt);
+			*done ||
+			test_bit(FORCE_DETACH, &device->flags) ||
+			test_bit(ABORT_MDIO, &device->flags),
+			dt);
+
 	if (dt == 0) {
 		drbd_err(device, "meta-data IO operation timed out\n");
 		drbd_handle_io_error(device, DRBD_FORCE_DETACH);
@@ -136,8 +144,7 @@ int drbd_md_sync_page_io(struct drbd_device *device, struct drbd_backing_dev *bd
 	D_ASSERT(device, atomic_read(&device->md_io.in_use) == 1);
 
 	if (!bdev->md_bdev) {
-		if (drbd_ratelimit())
-			drbd_err(device, "bdev->md_bdev==NULL\n");
+		drbd_err_ratelimit(device, "bdev->md_bdev==NULL\n");
 		return -EIO;
 	}
 
@@ -202,8 +209,7 @@ find_active_resync_extent(struct get_activity_log_ref_ctx *al_ctx)
 	return NULL;
 }
 
-void
-set_bme_priority(struct get_activity_log_ref_ctx *al_ctx)
+static void set_bme_priority(struct get_activity_log_ref_ctx *al_ctx)
 {
 	struct drbd_peer_device *peer_device;
 	struct lc_element *tmp;
@@ -430,10 +436,7 @@ static int __al_write_transaction(struct drbd_device *device, struct al_transact
 	if (drbd_bm_write_hinted(device))
 		err = -EIO;
 	else {
-		bool write_al_updates;
-		rcu_read_lock();
-		write_al_updates = rcu_dereference(device->ldev->disk_conf)->al_updates;
-		rcu_read_unlock();
+		bool write_al_updates = !(device->ldev->md.flags & MDF_AL_DISABLED);
 		if (write_al_updates) {
 			ktime_aggregate_delta(device, start_kt, al_mid_kt);
 			if (drbd_md_sync_page_io(device, device->ldev, sector, REQ_OP_WRITE)) {
@@ -531,11 +534,7 @@ void drbd_al_begin_io_commit(struct drbd_device *device)
 		/* Double check: it may have been committed by someone else
 		 * while we were waiting for the lock. */
 		if (device->act_log->pending_changes) {
-			bool write_al_updates;
-
-			rcu_read_lock();
-			write_al_updates = rcu_dereference(device->ldev->disk_conf)->al_updates;
-			rcu_read_unlock();
+			bool write_al_updates = !(device->ldev->md.flags & MDF_AL_DISABLED);
 
 			if (write_al_updates)
 				al_write_transaction(device);
@@ -563,6 +562,10 @@ static bool put_actlog(struct drbd_device *device, unsigned int first, unsigned
 	spin_lock_irqsave(&device->al_lock, flags);
 	for (enr = first; enr <= last; enr++) {
 		extent = lc_find(device->act_log, enr);
+		/* Yes, this masks a bug elsewhere.  However, during normal
+		 * operation this is harmless, so no need to crash the kernel
+		 * by the BUG_ON(refcount == 0) in lc_put().
+		 */
 		if (!extent || extent->refcnt == 0) {
 			drbd_err(device, "al_complete_io() called on inactive extent %u\n", enr);
 			continue;
@@ -640,40 +643,22 @@ int drbd_al_begin_io_for_peer(struct drbd_peer_device *peer_device, struct drbd_
 
 int drbd_al_begin_io_nonblock(struct drbd_device *device, struct drbd_interval *i)
 {
-	struct lru_cache *al = device->act_log;
 	struct bm_extent *bm_ext;
 	/* for bios crossing activity log extent boundaries,
 	 * we may need to activate two extents in one go */
 	unsigned first = i->sector >> (AL_EXTENT_SHIFT-9);
 	unsigned last = i->size == 0 ? first : (i->sector + (i->size >> 9) - 1) >> (AL_EXTENT_SHIFT-9);
-	unsigned nr_al_extents;
-	unsigned available_update_slots;
 	struct get_activity_log_ref_ctx al_ctx = { .device = device, };
 	unsigned enr;
 
-	D_ASSERT(device, first <= last);
-
-	nr_al_extents = 1 + last - first; /* worst case: all touched extends are cold. */
-	available_update_slots = min(al->nr_elements - al->used,
-				al->max_pending_changes - al->pending_changes);
-
-	/* We want all necessary updates for a given request within the same transaction
-	 * We could first check how many updates are *actually* needed,
-	 * and use that instead of the worst-case nr_al_extents */
-	if (available_update_slots < nr_al_extents) {
-		/* Too many activity log extents are currently "hot".
-		 *
-		 * If we have accumulated pending changes already,
-		 * we made progress.
-		 *
-		 * If we cannot get even a single pending change through,
-		 * stop the fast path until we made some progress,
-		 * or requests to "cold" extents could be starved. */
-		if (!al->pending_changes)
-			set_bit(__LC_STARVING, &device->act_log->flags);
-		return -ENOBUFS;
+	if (i->partially_in_al_next_enr) {
+		D_ASSERT(device, first < i->partially_in_al_next_enr);
+		D_ASSERT(device, last >= i->partially_in_al_next_enr);
+		first = i->partially_in_al_next_enr;
 	}
 
+	D_ASSERT(device, first <= last);
+
 	/* Is resync active in this area? */
 	for (enr = first; enr <= last; enr++) {
 		al_ctx.enr = enr;
@@ -686,14 +671,21 @@ int drbd_al_begin_io_nonblock(struct drbd_device *device, struct drbd_interval *
 		}
 	}
 
-	/* Checkout the refcounts.
-	 * Given that we checked for available elements and update slots above,
-	 * this has to be successful. */
+	/* Try to checkout the refcounts. */
 	for (enr = first; enr <= last; enr++) {
 		struct lc_element *al_ext;
 		al_ext = lc_get_cumulative(device->act_log, enr);
-		if (!al_ext)
-			drbd_err(device, "LOGIC BUG for enr=%u\n", enr);
+
+		if (!al_ext) {
+			/* Did not work. We may have exhausted the possible
+			 * changes per transaction. Or raced with someone
+			 * "locking" it against changes.
+			 * Remember where to continue from.
+			 */
+			if (enr > first)
+				i->partially_in_al_next_enr = enr;
+			return -ENOBUFS;
+		}
 	}
 	return 0;
 }
@@ -1028,19 +1020,30 @@ static bool update_rs_extent(struct drbd_peer_device *peer_device,
 
 void drbd_advance_rs_marks(struct drbd_peer_device *peer_device, unsigned long still_to_go)
 {
-	unsigned long now = jiffies;
-	unsigned long last = peer_device->rs_mark_time[peer_device->rs_last_mark];
-	int next = (peer_device->rs_last_mark + 1) % DRBD_SYNC_MARKS;
-	if (time_after_eq(now, last + DRBD_SYNC_MARK_STEP)) {
-		if (peer_device->rs_mark_left[peer_device->rs_last_mark] != still_to_go &&
-		    peer_device->repl_state[NOW] != L_PAUSED_SYNC_T &&
-		    peer_device->repl_state[NOW] != L_PAUSED_SYNC_S) {
-			peer_device->rs_mark_time[next] = now;
-			peer_device->rs_mark_left[next] = still_to_go;
-			peer_device->rs_last_mark = next;
-		}
-		drbd_peer_device_post_work(peer_device, RS_PROGRESS);
+	unsigned long now;
+	int next;
+
+	/* report progress and advance marks only if we made progress */
+	if (peer_device->rs_mark_left[peer_device->rs_last_mark] == still_to_go)
+		return;
+
+	/* report progress and advance marks at most once every DRBD_SYNC_MARK_STEP (3 seconds) */
+	now = jiffies;
+	if (!time_after_eq(now, peer_device->rs_last_progress_report_ts + DRBD_SYNC_MARK_STEP))
+		return;
+
+	/* Do not advance marks if we are "paused" */
+	if (peer_device->repl_state[NOW] != L_PAUSED_SYNC_T &&
+	    peer_device->repl_state[NOW] != L_PAUSED_SYNC_S) {
+		next = (peer_device->rs_last_mark + 1) % DRBD_SYNC_MARKS;
+		peer_device->rs_mark_time[next] = now;
+		peer_device->rs_mark_left[next] = still_to_go;
+		peer_device->rs_last_mark = next;
 	}
+
+	/* But still report progress even if paused. */
+	peer_device->rs_last_progress_report_ts = now;
+	drbd_peer_device_post_work(peer_device, RS_PROGRESS);
 }
 
 /* It is called lazy update, so don't do write-out too often. */
@@ -1151,7 +1154,7 @@ static bool plausible_request_size(int size)
 
 /* clear the bit corresponding to the piece of storage in question:
  * size byte of data starting from sector.  Only clear a bits of the affected
- * one ore more _aligned_ BM_BLOCK_SIZE blocks.
+ * one or more _aligned_ BM_BLOCK_SIZE blocks.
  *
  * called by worker on L_SYNC_TARGET and receiver on SyncSource.
  *
@@ -1232,9 +1235,10 @@ bool drbd_set_sync(struct drbd_device *device, sector_t sector, int size,
 		   unsigned long bits, unsigned long mask)
 {
 	long set_start, set_end, clear_start, clear_end;
-	sector_t esector, nr_sectors;
-	bool set = false;
 	struct drbd_peer_device *peer_device;
+	sector_t esector, nr_sectors;
+	unsigned long irq_flags;
+	bool set = false;
 
 	mask &= (1 << device->bitmap->bm_max_peers) - 1;
 
@@ -1269,6 +1273,7 @@ bool drbd_set_sync(struct drbd_device *device, sector_t sector, int size,
 	else
 		clear_end = BM_SECT_TO_BIT(esector + 1) - 1;
 
+	spin_lock_irqsave(&device->bitmap->bm_all_slots_lock, irq_flags);
 	rcu_read_lock();
 	for_each_peer_device_rcu(peer_device, device) {
 		int bitmap_index = peer_device->bitmap_index;
@@ -1298,7 +1303,7 @@ bool drbd_set_sync(struct drbd_device *device, sector_t sector, int size,
 						   clear_start, clear_end);
 		}
 	}
-
+	spin_unlock_irqrestore(&device->bitmap->bm_all_slots_lock, irq_flags);
 out:
 	put_ldev(device);
 
@@ -1360,8 +1365,12 @@ static int _is_in_al(struct drbd_device *device, unsigned int enr)
 
 /**
  * drbd_rs_begin_io() - Gets an extent in the resync LRU cache and sets it to BME_LOCKED
+ * @peer_device: DRBD peer device.
+ * @sector: The sector number.
  *
- * This functions sleeps on al_wait. Returns 0 on success, -EINTR if interrupted.
+ * This functions sleeps on al_wait.
+ *
+ * Returns: %0 on success, -EINTR if interrupted.
  */
 int drbd_rs_begin_io(struct drbd_peer_device *peer_device, sector_t sector)
 {
@@ -1409,9 +1418,14 @@ retry:
 
 /**
  * drbd_try_rs_begin_io() - Gets an extent in the resync LRU cache, does not sleep
+ * @peer_device: DRBD device.
+ * @sector:	The sector number.
+ * @throttle:	If true, allow to slow down the resync in case of heavy application IO
  *
  * Gets an extent in the resync LRU cache, sets it to BME_NO_WRITES, then
- * tries to set it to BME_LOCKED. Returns 0 upon success, and -EAGAIN
+ * tries to set it to BME_LOCKED.
+ *
+ * Returns: %0 upon success, and -EAGAIN
  * if there is still application IO going on in this area.
  */
 int drbd_try_rs_begin_io(struct drbd_peer_device *peer_device, sector_t sector, bool throttle)
@@ -1555,16 +1569,16 @@ void drbd_rs_complete_io(struct drbd_peer_device *peer_device, sector_t sector)
 	bm_ext = e ? lc_entry(e, struct bm_extent, lce) : NULL;
 	if (!bm_ext) {
 		spin_unlock_irqrestore(&device->al_lock, flags);
-		if (drbd_ratelimit())
-			drbd_err(device, "drbd_rs_complete_io() called, but extent not found\n");
+		drbd_err_ratelimit(device,
+			"%s called, but extent not found\n", __func__);
 		return;
 	}
 
 	if (bm_ext->lce.refcnt == 0) {
 		spin_unlock_irqrestore(&device->al_lock, flags);
-		drbd_err(device, "drbd_rs_complete_io(,%llu [=%u]) called, "
-		    "but refcnt is 0!?\n",
-		    (unsigned long long)sector, enr);
+		drbd_err(device,
+			"%s(,%llu [=%u]) called, but refcnt is 0!?\n",
+			__func__, (unsigned long long)sector, enr);
 		return;
 	}
 
@@ -1579,6 +1593,7 @@ void drbd_rs_complete_io(struct drbd_peer_device *peer_device, sector_t sector)
 
 /**
  * drbd_rs_cancel_all() - Removes all extents from the resync LRU (even BME_LOCKED)
+ * @peer_device: DRBD peer device.
  */
 void drbd_rs_cancel_all(struct drbd_peer_device *peer_device)
 {
@@ -1597,8 +1612,9 @@ void drbd_rs_cancel_all(struct drbd_peer_device *peer_device)
 
 /**
  * drbd_rs_del_all() - Gracefully remove all extents from the resync LRU
+ * @peer_device: DRBD peer device.
  *
- * Returns 0 upon success, -EAGAIN if at least one reference count was
+ * Returns: %0 upon success, -EAGAIN if at least one reference count was
  * not zero.
  */
 int drbd_rs_del_all(struct drbd_peer_device *peer_device)
