@@ -116,7 +116,7 @@ bm_print_lock_info(struct drbd_device *device, unsigned int bitmap_index, enum b
 	};
 
 	struct drbd_bitmap *b = device->bitmap;
-	if (!drbd_ratelimit())
+	if (!drbd_device_ratelimit(device, GENERIC))
 		return;
 	drbd_err(device, "FIXME %s[%d] op %s, bitmap locked for '%s' by %s[%d]\n",
 		 current->comm, task_pid_nr(current),
@@ -139,7 +139,7 @@ bm_print_lock_info(struct drbd_device *device, unsigned int bitmap_index, enum b
  */
 static void
 _drbd_bm_lock(struct drbd_device *device, struct drbd_peer_device *peer_device,
-	      char *why, enum bm_flag flags)
+	      const char *why, enum bm_flag flags)
 {
 	struct drbd_bitmap *b = device->bitmap;
 	int trylock_failed;
@@ -173,7 +173,7 @@ _drbd_bm_lock(struct drbd_device *device, struct drbd_peer_device *peer_device,
 	b->bm_locked_peer = peer_device;
 }
 
-void drbd_bm_lock(struct drbd_device *device, char *why, enum bm_flag flags)
+void drbd_bm_lock(struct drbd_device *device, const char *why, enum bm_flag flags)
 {
 	_drbd_bm_lock(device, NULL, why, flags);
 }
@@ -404,6 +404,7 @@ struct drbd_bitmap *drbd_bm_alloc(void)
 		return NULL;
 
 	spin_lock_init(&b->bm_lock);
+	spin_lock_init(&b->bm_all_slots_lock);
 	mutex_init(&b->bm_change);
 	init_waitqueue_head(&b->bm_io_wait);
 
@@ -476,6 +477,36 @@ static void bm_unmap(struct drbd_bitmap *bitmap, void *addr)
 		kunmap_atomic(addr);
 }
 
+
+/*
+ * find_next_bit() and find_next_zero_bit() expect an (unsigned long *),
+ * and will dereference it.
+ * When scanning our bitmap, we are interested in 32bit words of it.
+ * The "current 32 bit word pointer" may point to the last 32 bits in a page.
+ * For 64bit long, if the page after the current page is not mapped,
+ * this causes "page fault - not-present page".
+ * Duplicate the "fast path" of these functions,
+ * simplified for "size: 32, offset: 0".
+ * Little endian arch: le32_to_cpu is a no-op.
+ * Big endian arch: le32_to_cpu moves the least significant 32 bits around.
+ * __ffs / ffz do an implicit cast to (unsignd long). On 64bit, that fills up
+ * the most significant bits with 0; we are not interested in those anyways.
+ */
+static inline unsigned long find_next_bit_le32(const __le32 *addr)
+{
+	uint32_t val = *addr;
+
+	return val ? __ffs(le32_to_cpu(val)) : 32;
+}
+
+static inline unsigned long find_next_zero_bit_le32(const __le32 *addr)
+{
+	uint32_t val = *addr;
+
+	return val == ~0U ? 32 : ffz(le32_to_cpu(val));
+}
+
+
 static __always_inline unsigned long
 ____bm_op(struct drbd_device *device, unsigned int bitmap_index, unsigned long start, unsigned long end,
 	 enum bitmap_operations op, __le32 *buffer)
@@ -576,14 +607,18 @@ ____bm_op(struct drbd_device *device, unsigned int bitmap_index, unsigned long s
 				*buffer++ = *p;
 				break;
 			case BM_OP_FIND_BIT:
-				count = find_next_bit_le(addr, bit_in_page + 32, bit_in_page);
-				if (count < bit_in_page + 32)
+				count = find_next_bit_le32(p);
+				if (count < 32) {
+					count += bit_in_page;
 					goto found;
+				}
 				break;
 			case BM_OP_FIND_ZERO_BIT:
-				count = find_next_zero_bit_le(addr, bit_in_page + 32, bit_in_page);
-				if (count < bit_in_page + 32)
+				count = find_next_zero_bit_le32(p);
+				if (count < 32) {
+					count += bit_in_page;
 					goto found;
+				}
 				break;
 			}
 			start += 32;
@@ -1096,7 +1131,7 @@ static void drbd_bm_endio(struct bio *bio)
 		bm_set_page_io_err(b->bm_pages[idx]);
 		/* Not identical to on disk version of it.
 		 * Is BM_PAGE_IO_ERROR enough? */
-		if (drbd_ratelimit())
+		if (drbd_device_ratelimit(device, BACKEND))
 			drbd_err(device, "IO ERROR %d on bitmap page idx %u\n",
 				 status, idx);
 	} else {
@@ -1157,7 +1192,7 @@ static void bm_page_io_async(struct drbd_bm_aio_ctx *ctx, int page_nr) __must_ho
 		else
 			len = PAGE_SIZE;
 	} else {
-		if (drbd_ratelimit()) {
+		if (drbd_device_ratelimit(device, METADATA)) {
 			drbd_err(device, "Invalid offset during on-disk bitmap access: "
 				 "page idx %u, sector %llu\n", page_nr, (unsigned long long) on_disk_sector);
 		}
@@ -1285,7 +1320,8 @@ static int bm_rw_range(struct drbd_device *device,
 
 	now = jiffies;
 
-	/* let the layers below us try to merge these bios... */
+	blk_start_plug(&ctx->bm_aio_plug);
+	/* implicit unplug if scheduled for whatever reason */
 
 	if (flags & BM_AIO_READ) {
 		for (i = start_page; i <= end_page; i++) {
@@ -1334,6 +1370,8 @@ static int bm_rw_range(struct drbd_device *device,
 			cond_resched();
 		}
 	}
+	/* explicit unplug, we are done submitting */
+	blk_finish_plug(&ctx->bm_aio_plug);
 
 	/*
 	 * We initialize ctx->in_flight to one to make sure drbd_bm_endio
@@ -1348,8 +1386,8 @@ static int bm_rw_range(struct drbd_device *device,
 	} else
 		kref_put(&ctx->kref, &drbd_bm_aio_ctx_destroy);
 
-	/* summary for global bitmap IO */
-	if (flags == 0 && count) {
+	/* summary stats for global bitmap IO */
+	if ((flags & BM_AIO_NO_STATS) == 0 && count) {
 		unsigned int ms = jiffies_to_msecs(jiffies - now);
 		if (ms > 5) {
 			drbd_info(device, "bitmap %s of %u pages took %u ms\n",
@@ -1368,10 +1406,13 @@ static int bm_rw_range(struct drbd_device *device,
 		err = -EIO; /* Disk timeout/force-detach during IO... */
 
 	if (flags & BM_AIO_READ) {
+		unsigned int ms;
 		now = jiffies;
 		bm_count_bits(device);
-		drbd_info(device, "recounting of set bits took additional %ums\n",
-		     jiffies_to_msecs(jiffies - now));
+		ms = jiffies_to_msecs(jiffies - now);
+		/* If we can count quickly, there is no need to report this either */
+		if (ms > 3)
+			drbd_info(device, "recounting of set bits took additional %ums\n", ms);
 	}
 
 	kref_put(&ctx->kref, &drbd_bm_aio_ctx_destroy);
@@ -1635,7 +1676,8 @@ void drbd_bm_copy_slot(struct drbd_device *device, unsigned int from_index, unsi
 	u32 data_word, *addr;
 
 	words32_total = bitmap->bm_words * sizeof(unsigned long) / sizeof(u32);
-	spin_lock_irq(&bitmap->bm_lock);
+	spin_lock_irq(&bitmap->bm_all_slots_lock);
+	spin_lock(&bitmap->bm_lock);
 
 	bitmap->bm_set[to_index] = 0;
 	current_page_nr = 0;
@@ -1648,10 +1690,12 @@ void drbd_bm_copy_slot(struct drbd_device *device, unsigned int from_index, unsi
 
 		if (current_page_nr != from_page_nr) {
 			bm_unmap(bitmap, addr);
-			spin_unlock_irq(&bitmap->bm_lock);
+			spin_unlock(&bitmap->bm_lock);
+			spin_unlock_irq(&bitmap->bm_all_slots_lock);
 			if (need_resched())
 				cond_resched();
-			spin_lock_irq(&bitmap->bm_lock);
+			spin_lock_irq(&bitmap->bm_all_slots_lock);
+			spin_lock(&bitmap->bm_lock);
 			current_page_nr = from_page_nr;
 			addr = bm_map(bitmap, current_page_nr);
 		}
@@ -1670,5 +1714,6 @@ void drbd_bm_copy_slot(struct drbd_device *device, unsigned int from_index, unsi
 	}
 	bm_unmap(bitmap, addr);
 
-	spin_unlock_irq(&bitmap->bm_lock);
+	spin_unlock(&bitmap->bm_lock);
+	spin_unlock_irq(&bitmap->bm_all_slots_lock);
 }
