@@ -207,7 +207,7 @@ void drbd_peer_request_endio(struct bio *bio)
 
 	blk_status_t status = bio->bi_status;
 
-	if (status && drbd_ratelimit())
+	if (status && drbd_device_ratelimit(device, BACKEND))
 		drbd_warn(device, "%s: error=%d s=%llus\n",
 				is_write ? (is_discard ? "discard" : "write")
 					: "read", status,
@@ -274,7 +274,7 @@ void drbd_request_endio(struct bio *bio)
 	 * though we still will complain noisily about it.
 	 */
 	if (unlikely(req->local_rq_state & RQ_LOCAL_ABORTED)) {
-		if (drbd_ratelimit())
+		if (drbd_device_ratelimit(device, BACKEND))
 			drbd_emerg(device, "delayed completion of aborted local request; disk-timeout may be too aggressive\n");
 
 		if (!status)
@@ -472,7 +472,7 @@ int w_resync_timer(struct drbd_work *w, int cancel)
 			unsigned long irq_flags;
 			begin_state_change(resource, &irq_flags, 0);
 			peer_device->resync_active[NEW] = false;
-			end_state_change(resource, &irq_flags);
+			end_state_change(resource, &irq_flags, "resync-inactive");
 		}
 		break;
 	}
@@ -749,16 +749,16 @@ static bool send_buffer_half_full(struct drbd_peer_device *peer_device)
 	bool half_full = false;
 
 	mutex_lock(&connection->mutex[DATA_STREAM]);
-	if (transport->ops->stream_ok(transport, DATA_STREAM)) {
+	if (transport->class->ops.stream_ok(transport, DATA_STREAM)) {
 		struct drbd_transport_stats transport_stats;
 		int queued, sndbuf;
 
-		transport->ops->stats(transport, &transport_stats);
+		transport->class->ops.stats(transport, &transport_stats);
 		queued = transport_stats.send_buffer_used;
 		sndbuf = transport_stats.send_buffer_size;
 		if (queued > sndbuf / 2) {
 			half_full = true;
-			transport->ops->hint(transport, DATA_STREAM, NOSPACE);
+			transport->class->ops.hint(transport, DATA_STREAM, NOSPACE);
 		}
 	} else {
 		half_full = true;
@@ -829,8 +829,7 @@ static int make_resync_request(struct drbd_peer_device *peer_device, int cancel)
 		/* If a P_RS_CANCEL_AHEAD on control socket overtook the
 		 * already queued data and state change to Ahead/Behind,
 		 * don't add more resync requests, just wait it out. */
-		if (drbd_ratelimit())
-			drbd_info(peer_device, "peer pulled ahead during resync\n");
+		drbd_info_ratelimit(peer_device, "peer pulled ahead during resync\n");
 		return 0;
 	}
 
@@ -1155,7 +1154,7 @@ static void resync_again(struct drbd_device *device, u64 source_m, u64 target_m)
 				peer_device->resync_again--;
 				begin_state_change_locked(device->resource, CS_VERBOSE);
 				__change_repl_state(peer_device, new_repl_state);
-				end_state_change_locked(device->resource);
+				end_state_change_locked(device->resource, "resync-again");
 			}
 		}
 	}
@@ -1225,7 +1224,7 @@ found:
 	}
 
 	drbd_send_uuids(peer_device, 0, 0);
-	drbd_start_resync(peer_device, L_SYNC_TARGET);
+	drbd_start_resync(peer_device, L_SYNC_TARGET, "resync-from-primary");
 }
 
 static void queue_resync_finished(struct drbd_peer_device *peer_device, enum drbd_disk_state new_peer_disk_state)
@@ -1428,7 +1427,7 @@ void drbd_resync_finished(struct drbd_peer_device *peer_device,
 	}
 
 out_unlock:
-	end_state_change_locked(device->resource);
+	end_state_change_locked(device->resource, "resync-finished");
 
 	put_ldev(device);
 
@@ -1499,8 +1498,7 @@ int w_e_end_data_req(struct drbd_work *w, int cancel)
 	if (likely((peer_req->flags & EE_WAS_ERROR) == 0)) {
 		err = drbd_send_block(peer_device, P_DATA_REPLY, peer_req);
 	} else {
-		if (drbd_ratelimit())
-			drbd_err(peer_device, "Sending NegDReply. sector=%llus.\n",
+		drbd_err_ratelimit(peer_device, "Sending NegDReply. sector=%llus.\n",
 			    (unsigned long long)peer_req->i.sector);
 
 		err = drbd_send_ack(peer_device, P_NEG_DREPLY, peer_req);
@@ -1626,16 +1624,23 @@ int w_e_end_rsdata_req(struct drbd_work *w, int cancel)
 	} else if (likely((peer_req->flags & EE_WAS_ERROR) == 0)) {
 		if (likely(peer_device->disk_state[NOW] >= D_INCONSISTENT)) {
 			err = drbd_rs_reply(peer_device, peer_req);
+		} else if (connection->feature_flags & DRBD_FF_RESYNC_DAGTAG) {
+			/*
+			 * We do not support DRBD_FF_RESYNC_DAGTAG, but our
+			 * peer does. Help that peer by sending P_RS_CANCEL
+			 * instead of just ignoring the request.
+			 */
+			drbd_err_ratelimit(peer_device,
+				"Sending P_RS_CANCEL, partner DISKLESS!\n");
+			err = drbd_send_ack(peer_device, P_RS_CANCEL, peer_req);
 		} else {
-			if (drbd_ratelimit())
-				drbd_err(peer_device, "Not sending resync reply, "
-						"partner DISKLESS!\n");
+			drbd_err_ratelimit(peer_device,
+				"Not sending resync reply, partner DISKLESS!\n");
 			err = 0;
 		}
 	} else {
-		if (drbd_ratelimit())
-			drbd_err(peer_device, "Sending NegRSDReply. sector %llus.\n",
-			    (unsigned long long)peer_req->i.sector);
+		drbd_err_ratelimit(peer_device, "Sending NegRSDReply. sector %llus.\n",
+		    (unsigned long long)peer_req->i.sector);
 
 		err = drbd_send_ack(peer_device, P_NEG_RS_DREPLY, peer_req);
 
@@ -1920,7 +1925,8 @@ static bool drbd_pause_after(struct drbd_device *device)
 			if (!__drbd_may_sync_now(other_peer_device))
 				__change_resync_susp_dependency(other_peer_device, true);
 		}
-		if (end_state_change_locked(other_device->resource) != SS_NOTHING_TO_DO)
+		if (end_state_change_locked(other_device->resource, "resync-after") !=
+				SS_NOTHING_TO_DO)
 			changed = true;
 	}
 	rcu_read_unlock();
@@ -1957,7 +1963,8 @@ static bool drbd_resume_next(struct drbd_device *device)
 			    __drbd_may_sync_now(other_peer_device))
 				__change_resync_susp_dependency(other_peer_device, false);
 		}
-		if (end_state_change_locked(other_device->resource) != SS_NOTHING_TO_DO)
+		if (end_state_change_locked(other_device->resource, "resync-after") !=
+				SS_NOTHING_TO_DO)
 			changed = true;
 	}
 	rcu_read_unlock();
@@ -2125,7 +2132,7 @@ static void do_start_resync(struct drbd_peer_device *peer_device)
 		return;
 	}
 
-	drbd_start_resync(peer_device, peer_device->start_resync_side);
+	drbd_start_resync(peer_device, peer_device->start_resync_side, "postponed-resync");
 	clear_bit(AHEAD_TO_SYNC_SOURCE, &peer_device->flags);
 }
 
@@ -2149,7 +2156,7 @@ static void handle_congestion(struct drbd_peer_device *peer_device)
 			else if (on_congestion == OC_DISCONNECT)
 				__change_cstate(peer_device->connection, C_DISCONNECTING);
 		}
-		end_state_change(resource, &irq_flags);
+		end_state_change(resource, &irq_flags, "congestion");
 	}
 	rcu_read_unlock();
 
@@ -2163,7 +2170,8 @@ static void handle_congestion(struct drbd_peer_device *peer_device)
  * This function might bring you directly into one of the
  * C_PAUSED_SYNC_* states.
  */
-void drbd_start_resync(struct drbd_peer_device *peer_device, enum drbd_repl_state side)
+void drbd_start_resync(struct drbd_peer_device *peer_device, enum drbd_repl_state side,
+		const char *tag)
 {
 	struct drbd_device *device = peer_device->device;
 	struct drbd_connection *connection = peer_device->connection;
@@ -2258,7 +2266,7 @@ skip_helper:
 		init_resync_stable_bits(peer_device);
 	finished_resync_pdsk = peer_device->resync_finished_pdsk;
 	peer_device->resync_finished_pdsk = D_UNKNOWN;
-	r = end_state_change_locked(device->resource);
+	r = end_state_change_locked(device->resource, tag);
 	repl_state = peer_device->repl_state[NOW];
 
 	if (repl_state < L_ESTABLISHED)
@@ -2339,7 +2347,7 @@ static void go_diskless(struct drbd_device *device)
 	}
 
 	drbd_md_sync_if_dirty(device);
-	change_disk_state(device, D_DISKLESS, CS_HARD, NULL);
+	change_disk_state(device, D_DISKLESS, CS_HARD, "go-diskless", NULL);
 }
 
 static int do_md_sync(struct drbd_device *device)
@@ -2349,41 +2357,6 @@ static int do_md_sync(struct drbd_device *device)
 	return 0;
 }
 
-void repost_up_to_date_fn(struct timer_list *t)
-{
-	struct drbd_resource *resource = from_timer(resource, t, repost_up_to_date_timer);
-	drbd_post_work(resource, TWOPC_AFTER_LOST_PEER);
-}
-
-static int try_become_up_to_date(struct drbd_resource *resource)
-{
-	enum drbd_state_rv rv;
-
-	/* Doing a two_phase_commit from worker context is only possible
-	 * if twopc_work is not queued. Let it get executed first.
-	 *
-	 * Avoid deadlock on state_sem, in case someone holds it while
-	 * waiting for the completion of some after-state-change work.
-	 */
-	if (list_empty(&resource->twopc_work.list)) {
-		if (down_trylock(&resource->state_sem))
-			goto repost;
-		rv = twopc_after_lost_peer(resource, CS_ALREADY_SERIALIZED |
-			CS_VERBOSE | CS_SERIALIZE | CS_DONT_RETRY);
-		up(&resource->state_sem);
-		if (rv == SS_TIMEOUT || rv == SS_CONCURRENT_ST_CHG)
-			goto repost;
-		clear_bit(TWOPC_AFTER_LOST_PEER_PENDING, &resource->flags);
-		wake_up_all(&resource->state_wait);
-		drbd_notify_peers_lost_primary(resource);
-	} else {
-	repost:
-		mod_timer(&resource->repost_up_to_date_timer, jiffies + HZ/10);
-	}
-
-	return 0;
-}
-
 /* only called from drbd_worker thread, no locking */
 void __update_timing_details(
 		struct drbd_thread_timing_details *tdp,
@@ -2498,9 +2471,6 @@ static void do_peer_device_work(struct drbd_peer_device *peer_device, const unsi
 		handle_congestion(peer_device);
 }
 
-#define DRBD_RESOURCE_WORK_MASK	\
-	(1UL << TWOPC_AFTER_LOST_PEER)
-
 #define DRBD_DEVICE_WORK_MASK	\
 	((1UL << GO_DISKLESS)	\
 	|(1UL << MD_SYNC)	\
@@ -2575,14 +2545,6 @@ static void do_unqueued_device_work(struct drbd_resource *resource)
 	rcu_read_unlock();
 }
 
-static void do_unqueued_resource_work(struct drbd_resource *resource)
-{
-	unsigned long todo = get_work_bits(DRBD_RESOURCE_WORK_MASK, &resource->flags);
-
-	if (test_bit(TWOPC_AFTER_LOST_PEER, &todo))
-		try_become_up_to_date(resource);
-}
-
 static bool dequeue_work_batch(struct drbd_work_queue *queue, struct list_head *work_list)
 {
 	spin_lock_irq(&queue->q_lock);
@@ -3062,15 +3024,14 @@ int drbd_worker(struct drbd_thread *thi)
 		drbd_thread_current_set_cpu(thi);
 
 		if (list_empty(&work_list)) {
-			bool w, r, d, p;
+			bool w, d, p;
 
 			update_worker_timing_details(resource, dequeue_work_batch);
 			wait_event_interruptible(resource->work.q_wait,
 				(w = dequeue_work_batch(&resource->work, &work_list),
-				 r = test_and_clear_bit(RESOURCE_WORK_PENDING, &resource->flags),
 				 d = test_and_clear_bit(DEVICE_WORK_PENDING, &resource->flags),
 				 p = test_and_clear_bit(PEER_DEVICE_WORK_PENDING, &resource->flags),
-				 w || r || d || p));
+				 w || d || p));
 
 			if (p) {
 				update_worker_timing_details(resource, do_unqueued_peer_device_work);
@@ -3081,10 +3042,6 @@ int drbd_worker(struct drbd_thread *thi)
 				update_worker_timing_details(resource, do_unqueued_device_work);
 				do_unqueued_device_work(resource);
 			}
-			if (r) {
-				update_worker_timing_details(resource, do_unqueued_resource_work);
-				do_unqueued_resource_work(resource);
-			}
 		}
 
 		if (signal_pending(current)) {
@@ -3109,10 +3066,6 @@ int drbd_worker(struct drbd_thread *thi)
 	}
 
 	do {
-		if (test_and_clear_bit(RESOURCE_WORK_PENDING, &resource->flags)) {
-			update_worker_timing_details(resource, do_unqueued_resource_work);
-			do_unqueued_resource_work(resource);
-		}
 		if (test_and_clear_bit(DEVICE_WORK_PENDING, &resource->flags)) {
 			update_worker_timing_details(resource, do_unqueued_device_work);
 			do_unqueued_device_work(resource);
