@@ -17,6 +17,7 @@
 #include <linux/net.h>
 #include <linux/tcp.h>
 #include <linux/highmem.h>
+#include <linux/bio.h>
 #include <linux/drbd_genl_api.h>
 #include <linux/drbd_config.h>
 #include "drbd_protocol.h"
@@ -46,7 +47,6 @@ struct buffer {
 
 struct drbd_tcp_transport {
 	struct drbd_transport transport; /* Must be first! */
-	spinlock_t paths_lock;
 	unsigned long flags;
 	struct socket *stream[2];
 	struct buffer rbuf[2];
@@ -77,11 +77,16 @@ struct dtt_path {
 
 static int dtt_init(struct drbd_transport *transport);
 static void dtt_free(struct drbd_transport *transport, enum drbd_tr_free_op free_op);
+static int dtt_init_listener(struct drbd_transport *transport, const struct sockaddr *addr,
+			     struct net *net, struct drbd_listener *drbd_listener);
+static void dtt_destroy_listener(struct drbd_listener *generic_listener);
+static int dtt_prepare_connect(struct drbd_transport *transport);
 static int dtt_connect(struct drbd_transport *transport);
+static void dtt_finish_connect(struct drbd_transport *transport);
 static int dtt_recv(struct drbd_transport *transport, enum drbd_stream stream, void **buf, size_t size, int flags);
 static int dtt_recv_pages(struct drbd_transport *transport, struct drbd_page_chain_head *chain, size_t size);
 static void dtt_stats(struct drbd_transport *transport, struct drbd_transport_stats *stats);
-static void dtt_net_conf_change(struct drbd_transport *transport, struct net_conf *new_net_conf);
+static int dtt_net_conf_change(struct drbd_transport *transport, struct net_conf *new_net_conf);
 static void dtt_set_rcvtimeo(struct drbd_transport *transport, enum drbd_stream stream, long timeout);
 static long dtt_get_rcvtimeo(struct drbd_transport *transport, enum drbd_stream stream);
 static int dtt_send_page(struct drbd_transport *transport, enum drbd_stream, struct page *page,
@@ -91,83 +96,48 @@ static bool dtt_stream_ok(struct drbd_transport *transport, enum drbd_stream str
 static bool dtt_hint(struct drbd_transport *transport, enum drbd_stream stream, enum drbd_tr_hints hint);
 static void dtt_debugfs_show(struct drbd_transport *transport, struct seq_file *m);
 static void dtt_update_congested(struct drbd_tcp_transport *tcp_transport);
-static int dtt_add_path(struct drbd_transport *, struct drbd_path *path);
-static int dtt_remove_path(struct drbd_transport *, struct drbd_path *);
+static int dtt_add_path(struct drbd_path *path);
+static bool dtt_may_remove_path(struct drbd_path *);
+static void dtt_remove_path(struct drbd_path *);
 
 static struct drbd_transport_class tcp_transport_class = {
 	.name = "tcp",
 	.instance_size = sizeof(struct drbd_tcp_transport),
 	.path_instance_size = sizeof(struct dtt_path),
 	.listener_instance_size = sizeof(struct dtt_listener),
+	.ops = (struct drbd_transport_ops) {
+		.init = dtt_init,
+		.free = dtt_free,
+		.init_listener = dtt_init_listener,
+		.release_listener = dtt_destroy_listener,
+		.prepare_connect = dtt_prepare_connect,
+		.connect = dtt_connect,
+		.finish_connect = dtt_finish_connect,
+		.recv = dtt_recv,
+		.recv_pages = dtt_recv_pages,
+		.stats = dtt_stats,
+		.net_conf_change = dtt_net_conf_change,
+		.set_rcvtimeo = dtt_set_rcvtimeo,
+		.get_rcvtimeo = dtt_get_rcvtimeo,
+		.send_page = dtt_send_page,
+		.send_zc_bio = dtt_send_zc_bio,
+		.stream_ok = dtt_stream_ok,
+		.hint = dtt_hint,
+		.debugfs_show = dtt_debugfs_show,
+		.add_path = dtt_add_path,
+		.may_remove_path = dtt_may_remove_path,
+		.remove_path = dtt_remove_path,
+	},
 	.module = THIS_MODULE,
-	.init = dtt_init,
 	.list = LIST_HEAD_INIT(tcp_transport_class.list),
 };
 
-static struct drbd_transport_ops dtt_ops = {
-	.free = dtt_free,
-	.connect = dtt_connect,
-	.recv = dtt_recv,
-	.recv_pages = dtt_recv_pages,
-	.stats = dtt_stats,
-	.net_conf_change = dtt_net_conf_change,
-	.set_rcvtimeo = dtt_set_rcvtimeo,
-	.get_rcvtimeo = dtt_get_rcvtimeo,
-	.send_page = dtt_send_page,
-	.send_zc_bio = dtt_send_zc_bio,
-	.stream_ok = dtt_stream_ok,
-	.hint = dtt_hint,
-	.debugfs_show = dtt_debugfs_show,
-	.add_path = dtt_add_path,
-	.remove_path = dtt_remove_path,
-};
-
-/* Might restart iteration, if current element is removed from list!! */
-#define for_each_path_ref(path, transport)			\
-	for (path = __drbd_next_path_ref(NULL, transport);	\
-	     path;						\
-	     path = __drbd_next_path_ref(path, transport))
-
-/* This is save as long you use list_del_init() everytime something is removed
-   from the list. */
-static struct drbd_path *__drbd_next_path_ref(struct drbd_path *drbd_path,
-					      struct drbd_transport *transport)
-{
-	struct drbd_tcp_transport *tcp_transport =
-		container_of(transport, struct drbd_tcp_transport, transport);
-
-	spin_lock(&tcp_transport->paths_lock);
-	if (!drbd_path) {
-		drbd_path = list_first_entry_or_null(&transport->paths, struct drbd_path, list);
-	} else {
-		bool in_list = !list_empty(&drbd_path->list);
-		kref_put(&drbd_path->kref, drbd_destroy_path);
-		if (in_list) {
-			/* Element still on the list, ref count can not drop to zero! */
-			if (list_is_last(&drbd_path->list, &transport->paths))
-				drbd_path = NULL;
-			else
-				drbd_path = list_next_entry(drbd_path, list);
-		} else {
-			/* No longer on the list, element might be freed already, restart from the start */
-			drbd_path = list_first_entry_or_null(&transport->paths, struct drbd_path, list);
-		}
-	}
-	if (drbd_path)
-		kref_get(&drbd_path->kref);
-	spin_unlock(&tcp_transport->paths_lock);
-
-	return drbd_path;
-}
-
 static int dtt_init(struct drbd_transport *transport)
 {
 	struct drbd_tcp_transport *tcp_transport =
 		container_of(transport, struct drbd_tcp_transport, transport);
 	enum drbd_stream i;
 
-	spin_lock_init(&tcp_transport->paths_lock);
-	tcp_transport->transport.ops = &dtt_ops;
 	tcp_transport->transport.class = &tcp_transport_class;
 	for (i = DATA_STREAM; i <= CONTROL_STREAM ; i++) {
 		void *buffer = (void *)__get_free_page(GFP_KERNEL);
@@ -208,28 +178,18 @@ static void dtt_free(struct drbd_transport *transport, enum drbd_tr_free_op free
 		}
 	}
 
-	for_each_path_ref(drbd_path, transport) {
-		bool was_established = drbd_path->established;
-		drbd_path->established = false;
-		if (free_op == DESTROY_TRANSPORT)
-			drbd_path_event(transport, drbd_path, true);
-		else if (was_established)
-			drbd_path_event(transport, drbd_path, false);
+	list_for_each_entry(drbd_path, &transport->paths, list) {
+		bool was_established = test_and_clear_bit(TR_ESTABLISHED, &drbd_path->flags);
+
+		if (free_op == CLOSE_CONNECTION && was_established)
+			drbd_path_event(transport, drbd_path);
 	}
 
 	if (free_op == DESTROY_TRANSPORT) {
-		struct drbd_path *tmp;
-
 		for (i = DATA_STREAM; i <= CONTROL_STREAM; i++) {
 			free_page((unsigned long)tcp_transport->rbuf[i].base);
 			tcp_transport->rbuf[i].base = NULL;
 		}
-		spin_lock(&tcp_transport->paths_lock);
-		list_for_each_entry_safe(drbd_path, tmp, &transport->paths, list) {
-			list_del_init(&drbd_path->list);
-			kref_put(&drbd_path->kref, drbd_destroy_path);
-		}
-		spin_unlock(&tcp_transport->paths_lock);
 	}
 }
 
@@ -416,8 +376,9 @@ static bool dtt_path_cmp_addr(struct dtt_path *path)
 	return memcmp(&drbd_path->my_addr, &drbd_path->peer_addr, addr_size) > 0;
 }
 
-static int dtt_try_connect(struct drbd_transport *transport, struct dtt_path *path, struct socket **ret_socket)
+static int dtt_try_connect(struct dtt_path *path, struct socket **ret_socket)
 {
+	struct drbd_transport *transport = path->path.transport;
 	const char *what;
 	struct socket *socket;
 	struct sockaddr_storage my_addr, peer_addr;
@@ -575,23 +536,23 @@ static bool dtt_connection_established(struct drbd_transport *transport,
 	good += dtt_socket_ok_or_free(socket1);
 	good += dtt_socket_ok_or_free(socket2);
 
-	if (good == 0)
+	if (good == 0) {
+		kref_put(&(*first_path)->path.kref, drbd_destroy_path);
 		*first_path = NULL;
+	}
 
 	return good == 2;
 }
 
 static struct dtt_path *dtt_wait_connect_cond(struct drbd_transport *transport)
 {
-	struct drbd_tcp_transport *tcp_transport =
-		container_of(transport, struct drbd_tcp_transport, transport);
 	struct drbd_listener *listener;
 	struct drbd_path *drbd_path;
 	struct dtt_path *path = NULL;
 	bool rv = false;
 
-	spin_lock(&tcp_transport->paths_lock);
-	list_for_each_entry(drbd_path, &transport->paths, list) {
+	rcu_read_lock();
+	list_for_each_entry_rcu(drbd_path, &transport->paths, list) {
 		path = container_of(drbd_path, struct dtt_path, path);
 		listener = drbd_path->listener;
 
@@ -602,7 +563,9 @@ static struct dtt_path *dtt_wait_connect_cond(struct drbd_transport *transport)
 		if (rv)
 			break;
 	}
-	spin_unlock(&tcp_transport->paths_lock);
+	if (rv)
+		kref_get(&path->path.kref);
+	rcu_read_unlock();
 
 	return rv ? path : NULL;
 }
@@ -642,6 +605,8 @@ static int dtt_wait_for_connect(struct drbd_transport *transport,
 	timeo += get_random_u32_below(2) ? timeo / 7 : -timeo / 7; /* 28.5% random jitter */
 
 retry:
+	if (path)
+		kref_put(&path->path.kref, drbd_destroy_path);
 	timeo = wait_event_interruptible_timeout(listener->wait,
 			(path = dtt_wait_connect_cond(transport)),
 			timeo);
@@ -660,8 +625,10 @@ retry:
 
 		s_estab = NULL;
 		err = kernel_accept(listener->s_listen, &s_estab, O_NONBLOCK);
-		if (err < 0)
+		if (err < 0) {
+			kref_put(&path->path.kref, drbd_destroy_path);
 			return err;
+		}
 
 		/* The established socket inherits the sk_state_change callback
 		   from the listening socket. */
@@ -712,6 +679,8 @@ retry:
 	}
 	spin_unlock_bh(&listener->listener.waiters_lock);
 	*socket = s_estab;
+	if (*ret_path)
+		kref_put(&(*ret_path)->path.kref, drbd_destroy_path);
 	*ret_path = path;
 	return 0;
 
@@ -775,9 +744,10 @@ static void dtt_destroy_listener(struct drbd_listener *generic_listener)
 	struct dtt_listener *listener =
 		container_of(generic_listener, struct dtt_listener, listener);
 
+	if (!listener->s_listen)
+		return;
 	unregister_state_change(listener->s_listen->sk, listener);
 	sock_release(listener->s_listen);
-	kfree(listener);
 }
 
 static int dtt_init_listener(struct drbd_transport *transport,
@@ -842,7 +812,6 @@ static int dtt_init_listener(struct drbd_transport *transport,
 	}
 
 	listener->listener.listen_addr = my_addr;
-	listener->listener.destroy = dtt_destroy_listener;
 	init_waitqueue_head(&listener->wait);
 
 	return 0;
@@ -871,44 +840,61 @@ static void dtt_cleanup_accepted_sockets(struct dtt_path *path)
 	}
 }
 
-static void dtt_put_listeners(struct drbd_transport *transport)
+static void dtt_finish_connect(struct drbd_transport *transport)
 {
 	struct drbd_tcp_transport *tcp_transport =
 		container_of(transport, struct drbd_tcp_transport, transport);
-	struct drbd_path *drbd_path;
+	struct dtt_path *path;
 
-	spin_lock(&tcp_transport->paths_lock);
 	clear_bit(DTT_CONNECTING, &tcp_transport->flags);
-	spin_unlock(&tcp_transport->paths_lock);
 
-	for_each_path_ref(drbd_path, transport) {
-		struct dtt_path *path = container_of(drbd_path, struct dtt_path, path);
-
-		drbd_put_listener(drbd_path);
+	list_for_each_entry(path, &transport->paths, path.list) {
+		drbd_put_listener(&path->path);
 		dtt_cleanup_accepted_sockets(path);
 	}
 }
 
-static struct dtt_path *dtt_next_path(struct drbd_tcp_transport *tcp_transport, struct dtt_path *path)
+static struct dtt_path *dtt_next_path(struct dtt_path *path, struct drbd_transport *transport)
 {
-	struct drbd_transport *transport = &tcp_transport->transport;
 	struct drbd_path *drbd_path;
 
-	spin_lock(&tcp_transport->paths_lock);
-	if (list_is_last(&path->path.list, &transport->paths))
-		drbd_path = list_first_entry(&transport->paths, struct drbd_path, list);
-	else
-		drbd_path = list_next_entry(&path->path, list);
-	spin_unlock(&tcp_transport->paths_lock);
+	drbd_path = __drbd_next_path_ref(path ? &path->path : NULL, transport);
 
-	return container_of(drbd_path, struct dtt_path, path);
+	/* Loop when we reach the end. */
+	if (!drbd_path)
+		drbd_path = __drbd_next_path_ref(NULL, transport);
+
+	return drbd_path ? container_of(drbd_path, struct dtt_path, path) : NULL;
+}
+
+static int dtt_prepare_connect(struct drbd_transport *transport)
+{
+	struct drbd_tcp_transport *tcp_transport =
+		container_of(transport, struct drbd_tcp_transport, transport);
+	struct dtt_path *path;
+	struct drbd_path *drbd_path;
+
+	list_for_each_entry(path, &transport->paths, path.list)
+		dtt_cleanup_accepted_sockets(path);
+
+	set_bit(DTT_CONNECTING, &tcp_transport->flags);
+
+	list_for_each_entry(drbd_path, &transport->paths, list) {
+		if (!drbd_path->listener) {
+			int err = drbd_get_listener(drbd_path);
+
+			if (err)
+				return err;
+		}
+	}
+
+	return 0;
 }
 
 static int dtt_connect(struct drbd_transport *transport)
 {
 	struct drbd_tcp_transport *tcp_transport =
 		container_of(transport, struct drbd_tcp_transport, transport);
-	struct drbd_path *drbd_path;
 	struct dtt_path *connect_to_path, *first_path = NULL;
 	struct socket *dsocket, *csocket;
 	struct net_conf *nc;
@@ -918,59 +904,34 @@ static int dtt_connect(struct drbd_transport *transport)
 	dsocket = NULL;
 	csocket = NULL;
 
-
-	for_each_path_ref(drbd_path, transport) {
-		struct dtt_path *path = container_of(drbd_path, struct dtt_path, path);
-
-		dtt_cleanup_accepted_sockets(path);
-	}
-
-	spin_lock(&tcp_transport->paths_lock);
-	set_bit(DTT_CONNECTING, &tcp_transport->flags);
-
-	err = -EDESTADDRREQ;
-	if (list_empty(&transport->paths)) {
-		spin_unlock(&tcp_transport->paths_lock);
+	connect_to_path = dtt_next_path(NULL, transport);
+	if (!connect_to_path) {
+		err = -EDESTADDRREQ;
 		goto out;
 	}
 
-	list_for_each_entry(drbd_path, &transport->paths, list) {
-		if (!drbd_path->listener) {
-			kref_get(&drbd_path->kref);
-			spin_unlock(&tcp_transport->paths_lock);
-			err = drbd_get_listener(transport, drbd_path, dtt_init_listener);
-			kref_put(&drbd_path->kref, drbd_destroy_path);
-			if (err)
-				goto out;
-			spin_lock(&tcp_transport->paths_lock);
-			drbd_path = list_first_entry_or_null(&transport->paths, struct drbd_path, list);
-			if (drbd_path)
-				continue;
-			else
-				break;
-		}
-	}
-
-	drbd_path = list_first_entry(&transport->paths, struct drbd_path, list);
-	connect_to_path = container_of(drbd_path, struct dtt_path, path);
-	spin_unlock(&tcp_transport->paths_lock);
-
 	do {
 		struct socket *s = NULL;
 
-		err = dtt_try_connect(transport, connect_to_path, &s);
+		err = dtt_try_connect(connect_to_path, &s);
 		if (err < 0 && err != -EAGAIN)
-			goto out;
+			goto out_release_sockets;
 
 		if (s) {
 			bool use_for_data;
 
-			if (first_path && first_path != connect_to_path) {
-				tr_info(transport, "initial paths crossed A - fail over\n");
-				dtt_socket_free(&dsocket);
-				dtt_socket_free(&csocket);
+			if (first_path) {
+				if (first_path != connect_to_path) {
+					tr_info(transport, "initial paths crossed A - fail over\n");
+					dtt_socket_free(&dsocket);
+					dtt_socket_free(&csocket);
+				}
+
+				kref_put(&first_path->path.kref, drbd_destroy_path);
+				first_path = NULL;
 			}
 
+			kref_get(&connect_to_path->path.kref);
 			first_path = connect_to_path;
 
 			if (!dsocket && !csocket) {
@@ -993,8 +954,17 @@ static int dtt_connect(struct drbd_transport *transport)
 				csocket = s;
 				dtt_send_first_packet(tcp_transport, csocket, P_INITIAL_META, CONTROL_STREAM);
 			}
-		} else if (!first_path)
-			connect_to_path = dtt_next_path(tcp_transport, connect_to_path);
+		} else if (!first_path) {
+			connect_to_path = dtt_next_path(connect_to_path, transport);
+
+			/*
+			 * The final path should not be removed while
+			 * connecting, but handle the case for robustness.
+			 */
+			err = -EDESTADDRREQ;
+			if (!connect_to_path)
+				goto out_release_sockets;
+		}
 
 		if (dtt_connection_established(transport, &dsocket, &csocket, &first_path))
 			break;
@@ -1003,17 +973,23 @@ retry:
 		s = NULL;
 		err = dtt_wait_for_connect(transport, connect_to_path->path.listener, &s, &connect_to_path);
 		if (err < 0 && err != -EAGAIN)
-			goto out;
+			goto out_release_sockets;
 
 		if (s) {
 			int fp = dtt_receive_first_packet(tcp_transport, s);
 
-			if (first_path && first_path != connect_to_path) {
-				tr_info(transport, "initial paths crossed P - fail over\n");
-				dtt_socket_free(&dsocket);
-				dtt_socket_free(&csocket);
+			if (first_path) {
+				if (first_path != connect_to_path) {
+					tr_info(transport, "initial paths crossed P - fail over\n");
+					dtt_socket_free(&dsocket);
+					dtt_socket_free(&csocket);
+				}
+
+				kref_put(&first_path->path.kref, drbd_destroy_path);
+				first_path = NULL;
 			}
 
+			kref_get(&connect_to_path->path.kref);
 			first_path = connect_to_path;
 
 			dtt_socket_ok_or_free(&dsocket);
@@ -1057,9 +1033,8 @@ randomize:
 	} while (!ok);
 
 	TR_ASSERT(transport, first_path == connect_to_path);
-	connect_to_path->path.established = true;
-	drbd_path_event(transport, &connect_to_path->path, false);
-	dtt_put_listeners(transport);
+	set_bit(TR_ESTABLISHED, &connect_to_path->path.flags);
+	drbd_path_event(transport, &connect_to_path->path);
 
 	dsocket->sk->sk_reuse = SK_CAN_REUSE; /* SO_REUSEADDR */
 	csocket->sk->sk_reuse = SK_CAN_REUSE; /* SO_REUSEADDR */
@@ -1105,33 +1080,38 @@ randomize:
 	if (drbd_keepintvl)
 		tcp_sock_set_keepintvl(dsocket->sk, drbd_keepintvl);
 
-	return 0;
+
+	err = 0;
+	goto out;
 
 out_eagain:
 	err = -EAGAIN;
 
-out:
-	dtt_put_listeners(transport);
+out_release_sockets:
+	dtt_socket_free(&dsocket);
+	dtt_socket_free(&csocket);
 
-	if (dsocket) {
-		kernel_sock_shutdown(dsocket, SHUT_RDWR);
-		sock_release(dsocket);
-	}
-	if (csocket) {
-		kernel_sock_shutdown(csocket, SHUT_RDWR);
-		sock_release(csocket);
-	}
+out:
+	if (first_path)
+		kref_put(&first_path->path.kref, drbd_destroy_path);
+	if (connect_to_path)
+		kref_put(&connect_to_path->path.kref, drbd_destroy_path);
 
 	return err;
 }
 
-static void dtt_net_conf_change(struct drbd_transport *transport, struct net_conf *new_net_conf)
+static int dtt_net_conf_change(struct drbd_transport *transport, struct net_conf *new_net_conf)
 {
 	struct drbd_tcp_transport *tcp_transport =
 		container_of(transport, struct drbd_tcp_transport, transport);
 	struct socket *data_socket = tcp_transport->stream[DATA_STREAM];
 	struct socket *control_socket = tcp_transport->stream[CONTROL_STREAM];
 
+	if (new_net_conf->tls) {
+		tr_warn(transport, "TLS not supported\n");
+		return -EINVAL;
+	}
+
 	if (data_socket) {
 		dtt_setbufsize(data_socket, new_net_conf->sndbuf_size, new_net_conf->rcvbuf_size);
 	}
@@ -1139,6 +1119,8 @@ static void dtt_net_conf_change(struct drbd_transport *transport, struct net_con
 	if (control_socket) {
 		dtt_setbufsize(control_socket, new_net_conf->sndbuf_size, new_net_conf->rcvbuf_size);
 	}
+
+	return 0;
 }
 
 static void dtt_set_rcvtimeo(struct drbd_transport *transport, enum drbd_stream stream, long timeout)
@@ -1193,18 +1175,22 @@ static int dtt_send_page(struct drbd_transport *transport, enum drbd_stream stre
 	struct drbd_tcp_transport *tcp_transport =
 		container_of(transport, struct drbd_tcp_transport, transport);
 	struct socket *socket = tcp_transport->stream[stream];
+	struct msghdr msg = { .msg_flags = msg_flags | MSG_NOSIGNAL | MSG_SPLICE_PAGES };
+	struct bio_vec bvec;
 	int len = size;
 	int err = -EIO;
 
 	if (!socket)
 		return -ENOTCONN;
 
-	msg_flags |= MSG_NOSIGNAL;
 	dtt_update_congested(tcp_transport);
 	do {
 		int sent;
 
-		sent = socket->ops->sendpage(socket, page, offset, len, msg_flags);
+		bvec_set_page(&bvec, page, len, offset);
+		iov_iter_bvec(&msg.msg_iter, ITER_SOURCE, &bvec, 1, len);
+
+		sent = sock_sendmsg(socket, &msg);
 		if (sent <= 0) {
 			if (sent == -EAGAIN) {
 				if (drbd_stream_send_timed_out(transport, stream))
@@ -1319,52 +1305,30 @@ static void dtt_debugfs_show(struct drbd_transport *transport, struct seq_file *
 
 }
 
-static int dtt_add_path(struct drbd_transport *transport, struct drbd_path *drbd_path)
+static int dtt_add_path(struct drbd_path *drbd_path)
 {
+	struct drbd_transport *transport = drbd_path->transport;
 	struct drbd_tcp_transport *tcp_transport =
 		container_of(transport, struct drbd_tcp_transport, transport);
 	struct dtt_path *path = container_of(drbd_path, struct dtt_path, path);
-	bool active;
 
-	drbd_path->established = false;
+	clear_bit(TR_ESTABLISHED, &drbd_path->flags);
 	INIT_LIST_HEAD(&path->sockets);
-retry:
-	active = test_bit(DTT_CONNECTING, &tcp_transport->flags);
-	if (!active && drbd_path->listener)
-		drbd_put_listener(drbd_path);
 
-	if (active && !drbd_path->listener) {
-		int err = drbd_get_listener(transport, drbd_path, dtt_init_listener);
-		if (err)
-			return err;
-	}
+	if (!test_bit(DTT_CONNECTING, &tcp_transport->flags))
+		return 0;
 
-	spin_lock(&tcp_transport->paths_lock);
-	if (active != test_bit(DTT_CONNECTING, &tcp_transport->flags)) {
-		spin_unlock(&tcp_transport->paths_lock);
-		goto retry;
-	}
-	list_add_tail(&drbd_path->list, &transport->paths);
-	spin_unlock(&tcp_transport->paths_lock);
-
-	return 0;
+	return drbd_get_listener(drbd_path);
 }
 
-static int dtt_remove_path(struct drbd_transport *transport, struct drbd_path *drbd_path)
+static bool dtt_may_remove_path(struct drbd_path *drbd_path)
 {
-	struct drbd_tcp_transport *tcp_transport =
-		container_of(transport, struct drbd_tcp_transport, transport);
-	struct dtt_path *path = container_of(drbd_path, struct dtt_path, path);
+	return !test_bit(TR_ESTABLISHED, &drbd_path->flags);
+}
 
-	if (drbd_path->established)
-		return -EBUSY;
-
-	spin_lock(&tcp_transport->paths_lock);
-	list_del_init(&drbd_path->list);
-	spin_unlock(&tcp_transport->paths_lock);
-	drbd_put_listener(&path->path);
-
-	return 0;
+static void dtt_remove_path(struct drbd_path *drbd_path)
+{
+	drbd_put_listener(drbd_path);
 }
 
 static int __init dtt_initialize(void)
